import React from 'react';
import { expect } from 'chai';
import { shallow } from 'enzyme';
import { Route } from 'react-router-dom';

import UniversalRoute from './UniversalRoute';

describe('UniversalRoute', () => {
  it('should show a loading element when no component is passed', () => {
    const props = {};
    const wrapper = shallow(<UniversalRoute {...props} />);

    expect(wrapper.html()).to.equal('<div>Loading</div>');
  });

  it('should render the Route when a component exists in props', () => {
    const props = {
      component: () => (<div>My component</div>),
    };
    const wrapper = shallow(<UniversalRoute {...props} />);

    expect(wrapper.state('asyncComponent')).to.equal(props.component);
    expect(wrapper.find(Route).length).to.equal(1);
  });

  it('should try to fetch a bundle if passed as prop', async () => {
    const myComponent = () => (<div>My component</div>);
    const props = {
      bundle: {
        load: () => Promise.resolve({ default: myComponent }),
      },
    };

    const wrapper = shallow(<UniversalRoute {...props} />);
    await wrapper.instance().componentDidMount();

    expect(wrapper.state('asyncComponent')).to.equal(myComponent);
  });
});
