//  @flow
import React, { Component } from 'react';
import { Route } from 'react-router-dom';

import RoutePreloader from './RoutePreloader';

type DefaultProps = {
  [string]: any,
};

type Props = {
  bundle: ?{ load: Function },
  component: ?Function,
};

type State = { asyncComponent: ?Function };

class UniversalRoute extends Component<DefaultProps, Props, State> {
  static defaultProps = {}

  constructor(props: Props) {
    super(props);
    this.state = {
      asyncComponent: props.component,
    };
  }

  state: State;

  componentDidMount() {
    this.resolveBundle();
  }

  resolveBundle = async () => {
    const { bundle } = this.props;
    if (bundle) {
      const resolved = await bundle.load();
      this.setState({
        asyncComponent: resolved.default,
      });
    }
  }

  render() {
    const { component, ...rest } = this.props;
    const { asyncComponent } = this.state;

    if (!asyncComponent) {
      return (<div>Loading</div>);
    }

    return (
      <Route {...rest} component={RoutePreloader(asyncComponent, rest)} />
    );
  }
}

export default UniversalRoute;
