//  @flow
import path from 'path';
import express from 'express';
import webpackDevMiddleware from 'webpack-dev-middleware';
import webpackHotMiddleware from 'webpack-hot-middleware';
import webpack from 'webpack';

import { getMessage } from './../../../tools/messaging';
import getWebpackConfig from './../../webpack/webpack.config';

import type { IConfig } from './../../../flow/IConfig';
import type { IApp } from './../../../flow/IApp';

export default (app: IApp, config: IConfig) => {
  const _app = app;
  const { isDevEnvironment, publicPath, distOutput } = config;

  try {
    if (!isDevEnvironment) {
      _app.use(publicPath, express.static(path.join(process.cwd(), distOutput, publicPath)));
    }

    if (isDevEnvironment) {
      const webpackConfig = getWebpackConfig(config);
      const compiler = webpack(webpackConfig);
      const hotMiddleware = webpackHotMiddleware(compiler);

      _app.locals.compiler = compiler;
      _app.locals.hotMiddleware = hotMiddleware;
      _app.use(publicPath, webpackDevMiddleware(compiler));
      _app.use(hotMiddleware);
    }
  } catch (error) {
    throw new Error(getMessage('error', { type: 'static', error }));
  }

  return _app;
};
