//  @flow
import express from 'express';
import type { $Request, $Response } from 'express';

import { getMessage } from './../tools/messaging';
import composedMiddleware, { applyStaticHandlers, errorHandler } from './middleware';

import hooks from './../tools/hooks';
import defaultConfig from './defaultConfig';

import type { IConfig } from './../flow/IConfig';
import type { IApp } from './../flow/IApp';

//  Run hooks before
//  starting application
hooks();

class Server {
  config: { [string]: any };
  app: IApp;

  constructor(config: IConfig): void {
    this.config = { ...defaultConfig, ...config };
    this.app = applyStaticHandlers(express(), this.config);
  }

  create(callback: ?Function) {
    const app = this.app;
    const config = this.config;
    const { port, diagnostics } = config;

    app.get(diagnostics.heartbeat, (req: $Request, res: $Response) => (
      res.send('OK')
    ));

    app.get(diagnostics.version, (req: $Request, res: $Response) => (
      res.send(process.env.APP_VERSION || '0.0.0')
    ));

    app.use(composedMiddleware(app, config), (req: $Request, res: $Response) => (
      res.send(app.locals.output)),
    );

    app.use(errorHandler);

    app.listen(port || 3000, () => {
      console.log(getMessage('welcome', config));
      if (callback && typeof callback === 'function') {
        callback();
      }
    });

    return app;
  }
}

export default Server;
